/*
 * (POP3Lite) Sabotage - 3lite POP3 Daemon (Sabotage)
 * Copyright (C) 2000, 2001 Gergely Nagy <8@free.bsd.hu>
 *
 * This file is part of POP3Lite.
 *
 * POP3Lite is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * POP3Lite is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

#ifdef HAVE_CONFIG_H
#	include <config.h>
#endif

#include <pop3lite.h>
#include <glib.h>
#include "md5-wrap.h"

#include <string.h>
#include <sys/types.h>
#include <unistd.h>

#ifdef DEBUG
#	include <syslog.h>
#endif

static const char rcsid[]="$Id: sabotage.c,v 1.9.2.1 2001/04/18 10:15:33 algernon Exp $";

int sabotage_LTX_module_init ( P3LControl *control );
int sabotage_LTX_module_done ( P3LControl *control );

static p3l_pop3_command B_sabotage_cmd_trans_retr;
static P3LControl_trans_init B_sabotage_uidl_trans_init;

static CommandResponse *sabotage_cmd_trans_retr ( P3LControl *control, char *args );
static CommandResponse *sabotage_uidl_trans_init ( P3LControl *control );

static CommandResponse *
sabotage_uidl_trans_init ( P3LControl *control )
{
	CommandResponse *res;
	unsigned long i;
	md5_context context;
	MailInfo *minfo;
	char *pid=g_strdup_printf ( "%u", (unsigned int) getpid () );

#ifdef DEBUG
	control->system->log ( control, LOG_DEBUG, "%s:%d: TRANSACTION state started",
			       __FILE__, __LINE__ );
#endif

	res = (*B_sabotage_uidl_trans_init) ( control );

	for ( i = 0; i < g_list_length ( control->msg_info ); i++ )
	{
		minfo = (MailInfo *) g_list_nth_data ( control->msg_info, i );
		__md5_init_ctx ( &context );
		__md5_process_bytes ( minfo->digest, strlen ( minfo->digest ), &context );
		__md5_process_bytes ( pid, strlen ( pid ), &context );
		__md5_finish_ctx ( &context, &(minfo->digest) ); 
	}

	return res;
}

static CommandResponse *
sabotage_cmd_trans_retr ( P3LControl *control, char *args )
{
	p3l_pop3_command cmd_dele;
	CommandResponse *res;

#ifdef DEBUG
	control->system->log ( control, LOG_DEBUG, "%s:%d: handling RETR",
			       __FILE__, __LINE__ );
#endif

	res = (*B_sabotage_cmd_trans_retr) ( control, args );
	cmd_dele = (p3l_pop3_command) g_hash_table_lookup ( control->trans_commands, "DELE" );
	(*cmd_dele) ( control, args );

	return res;
}

int
sabotage_LTX_module_init ( P3LControl *control )
{
	int r = -1;
#ifdef DEBUG
	control->system->log ( control, LOG_DEBUG, "%s:%d: init mod-Sabotage",
			       __FILE__, __LINE__ );
#endif

	if ( p3l_is_enabled ( P3L_GET_FIRST_OPTION ( "SABOTAGE.RETR" ) ) )
	{
		B_sabotage_cmd_trans_retr = (p3l_pop3_command)
			p3l_command_replace ( control->trans_commands, "RETR",
					      (gpointer) sabotage_cmd_trans_retr );
		r = 0;
	}

	if ( p3l_is_enabled ( P3L_GET_FIRST_OPTION ( "SABOTAGE.UIDL" ) ) )
	{
		B_sabotage_uidl_trans_init = control->trans_init;
		control->trans_init = sabotage_uidl_trans_init;

		r = 0;
	}

	return r;
}

int
sabotage_LTX_module_done ( P3LControl *control )
{
#ifdef DEBUG
	control->system->log ( control, LOG_DEBUG, "%s:%d: done mod-Sabotage",
			       __FILE__, __LINE__ );
#endif

	g_hash_table_insert ( control->trans_commands, "RETR",
			      (gpointer) B_sabotage_cmd_trans_retr );

	control->trans_init = B_sabotage_uidl_trans_init;

	return 0;
}
